/*

    Libre Media Server Audio - An Open source Media Server for arts and performing.
    (c) Criptomart - Santiago Noreña 2012-2024 <lms@criptomart.net>
    https://git.criptomart.net/libremediaserver

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "libremediaserver-audio.h"


libreMediaServerAudio::libreMediaServerAudio(QStringList args, QWidget *parent)
    : QMainWindow(parent)
{
    Q_UNUSED(args);
    qDebug() << VERSION;
    qDebug() << COPYRIGHT;
    qDebug() << LICENSE;
    ui.setupUi(this);
    this->setWindowTitle(VERSION);
    Settings *set = Settings::getInstance();
    set->readFile();
    m_mediaLibrary = new MediaLibrary;
    m_mediaLibrary->initMediaLibrary();
    m_aw = new AudioWidget;
    setCentralWidget(m_aw);
    m_dmxWidget = new dmxWidget(this);
    QDockWidget *topWidget = new QDockWidget(tr("Master"), this);
    topWidget->setAllowedAreas(Qt::TopDockWidgetArea);
    topWidget->setWidget(m_dmxWidget);
    addDockWidget(Qt::TopDockWidgetArea, topWidget);
    m_ola = new olaThread(this, set->getLayersNumber());
    Q_CHECK_PTR(m_ola);
    m_ola->blockSignals(true);
    connect(m_ola, SIGNAL (universeReceived(int)), m_dmxWidget, SLOT(updateWatchDMX(int)));
    connect(m_ola, SIGNAL(dmxOutput(int, int, int)), this, SLOT(dmxInput(int, int, int)));
    m_ola->registerUniverse();
    connect(ui.actionLaunch_OLA_Setup, SIGNAL(triggered()), this, SLOT(olasetup()));
    m_aw->startEngine();
    qDebug("Init Complete.");
    m_ola->blockSignals(false);
    m_ola->start(QThread::TimeCriticalPriority );
}

libreMediaServerAudio::~libreMediaServerAudio()
{
    m_ola->stop();
    m_aw->stopEngine();
}

void libreMediaServerAudio::olasetup()
{
    QWebView *view = new QWebView();
    view->load(QUrl("http://localhost:9090/ola.html"));
    view->show();
}

void libreMediaServerAudio::dmxInput(int layer, int channel, int value)
{
    if (layer > LAYER_CHANNELS)
        return;
    QString mediaFile = NULL;
    int aux;
    switch(channel){
    case DMX_FOLDER:
        aux = m_ola->getValue(layer, DMX_FILE);
        mediaFile = m_mediaLibrary->requestNewFile(value, aux);
        if (QFile::exists(mediaFile))
            m_aw->mediaLoaded(layer, mediaFile);
        break;
    case DMX_FILE:
        aux = m_ola->getValue(layer, DMX_FOLDER);
        mediaFile = m_mediaLibrary->requestNewFile(aux, value);
        if (QFile::exists(mediaFile))
            m_aw->mediaLoaded(layer, mediaFile);
        break;
    case VOLUME_COARSE:
    case VOLUME_FINE:
        m_aw->volChanged(layer, (value / 65025.0f));
        break;
    case PAN:
        m_aw->panChanged(layer, value);
        break;
    case PITCH:
        m_aw->pitchChanged(layer, value);
        break;
    case ENTRY_POINT_COARSE:
    case ENTRY_POINT_FINE:
        m_aw->entryPointChanged(layer, value);
        break;
    case PLAYBACK:
        if (value == 0)
            break;
        aux = value / 25;
        switch (aux) {
        case 0 :
            m_aw->playbackChanged(layer, PlayingOnce);
            break;
        case 1 :
            m_aw->playbackChanged(layer, Stopped);
            break;
        case 2 :
            m_aw->playbackChanged(layer, Paused);
            break;
        case 3 :
            m_aw->playbackChanged(layer, PlayingLoop);
            break;
        default :
            break;
        }
    default:
        break;
    }
}
